/*
  A simple program that utilizes an ultrasonic sensor to disribute a mini candy bar using 28byj-48 steppers with ULN2003 drivers.
  Written by David Tanner 2019
*/

//Include the Arduino Stepper Library
#include <Stepper.h>
// Defines that CS pin for the SD card module will go through pin 10.
#define SD_ChipSelectPin 10 
// SPI - Version: Latest
#include <SPI.h>
// SD - Version: 1.2.X for interfacing with SD card module
#include <SD.h>
// This library allows the program to play a .wav file off of the SD card.
#include"TMRpcm.h"

//rename function for easier use
TMRpcm sound;   



// Number of steps per internal motor revolution 
const float STEPS_PER_REV = 32; 
 
//  Amount of Gear Reduction
const float GEAR_RED = 64;

// Number of steps per geared output rotation
const float STEPS_PER_OUT_REV = STEPS_PER_REV * GEAR_RED;

// Number of steps required for dispensing
int StepsRequired;

// Holds the number of steps required for head turn
int headTurn;


// you can use this in your sketch if you'd like to retain a count of how many pieces have been dispensed.
int dispenseCount = 0;
String dispenseMessage = "Candy dispense Count = ";

//trigger and echo pins for HC-SR04 Ultrasonic sensor
const int trigPin = 4; 
const int echoPin = 2;

// Map the candy motor stepper pins to analog pins
const int motPin1 = A3; 
const int motPin2 = A2; 
const int motPin3 = A1; 
const int motPin4 = A0; 


//holds sound wave length and distance away for the ultrasonic sensor
float duration, distance; 

// Stepper schematic
// Connected to ULN2003 Motor Driver In1, In2, In3, In4 
// Pins entered in sequence 1-3-2-4 for proper step sequencing
Stepper headMotor(STEPS_PER_REV, 9, 7, 8, 6);
Stepper candyMotor(STEPS_PER_REV, motPin1, motPin3, motPin2, motPin4);

void setup()
{
Serial.begin(9600);
pinMode(motPin4, OUTPUT);
pinMode(motPin3, OUTPUT);
pinMode(motPin2, OUTPUT);
pinMode(motPin1, OUTPUT);
pinMode(5, OUTPUT);
digitalWrite(5, HIGH);
sound.speakerPin = 5;
sound.setVolume(5);
delay(2000);
head();
//play welcome audio
sound.play("hello.wav");
// Set ultrasonic pins as out or inputs
pinMode(trigPin, OUTPUT); 
pinMode(echoPin, INPUT); 
Serial.println("Ultrasonic pins set.");
delay(1000);

if (!SD.begin(SD_ChipSelectPin)) {
    Serial.println("initialization failed!");
    while (0);
  }
  Serial.println("initialization done.");
}

void loop()
{
delay(2);
// Listen to ultrasonic sensor for nearby objects
  digitalWrite(trigPin, LOW); 
  delayMicroseconds(2); 
  digitalWrite(trigPin, HIGH); 
  delayMicroseconds(10); 
  digitalWrite(trigPin, LOW);
  duration = pulseIn(echoPin, HIGH);
  distance = (duration*.343)/2;
  
  delay(500); 
  // Change the distance in mm based on how far you want the sensor to pick up
  if (distance <= 1200.00) {
        delay(1000);
        Serial.println("Dispensing..."); 
        dispense();
        //dispenseCount = ++dispenseCount;
        Serial.println("Complete!");
        //Serial.println(String(dispenseMessage) + dispenseCount); 

        //play treat audio
        treat();
        head();
        exitAudio();   
}
}

void dispense()
{
  // Run candy motor to dispense
  candyMotor.setSpeed(500);
  StepsRequired  =  340;
  candyMotor.step(StepsRequired);
  delay(2000);
}

void head()
{
  //rotate head back and forth
  headMotor.setSpeed(500);
  headTurn = 100;
  headMotor.step(50);
  delay(200);
  headMotor.step(headTurn * -1);
  delay(200);
  headMotor.step(50);
}

void welcome()
{
  // play welcome audio when called
  sound.play("hello.wav");
}

void invite()
{
  // play invite audio when called
  sound.play("invite.wav");
  
}

void treat()
{
  // play dispensing audio when called
  sound.play("treat.wav");
  
}

void exitAudio()
{
  // play exit audio when called
  sound.play("goodbye.wav");
}
